import type { ColorValue, StyleProp, TextStyle, ViewStyle } from 'react-native'
import type { SyntaxHighlighterProps } from 'src/types/shiki'
import type { ReactStyle } from '../../utils/style-transformer'
import React, { useMemo } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'
import { useSyntaxHighlighter } from '../../hooks/useSyntaxHighlighter'
import { getRNStylesFromShikiStyle } from '../../utils/style-transformer'

const monospaceFont = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface Styles {
  scrollView: ViewStyle
  lineContainer: ViewStyle
  token: TextStyle
  container: ViewStyle
}

const baseStyles = StyleSheet.create<Styles>({
  scrollView: {
    flex: 1,
    minHeight: 20,
  },
  lineContainer: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
  token: {
    fontFamily: monospaceFont,
  },
  container: {
    flex: 1,
  },
})

type FontWeight = 'normal' | 'bold' | '100' | '200' | '300' | '400' | '500' | '600' | '700' | '800' | '900'

interface TokenStyle extends Omit<TextStyle, 'color' | 'fontWeight'> {
  color?: ColorValue
  fontStyle?: 'normal' | 'italic'
  fontWeight?: FontWeight
  fontFamily?: string
}

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme }) => {
  const stylesheet = useMemo(() => {
    const styles = getRNStylesFromShikiStyle(theme as ReactStyle)
    return StyleSheet.create(styles)
  }, [theme])

  const { tokens } = useSyntaxHighlighter({
    text,
    language,
    languageId,
    theme,
  })

  const renderToken = (token: any, index: number, lineIndex: number) => {
    const styles: StyleProp<TokenStyle>[] = [baseStyles.token]

    if (stylesheet.base) {
      styles.push(stylesheet.base as unknown as TokenStyle)
    }

    const tokenScopes = Array.isArray(token.scope) ? token.scope : [token.scope]
    tokenScopes.forEach((scope: string) => {
      if (stylesheet[scope]) {
        styles.push(stylesheet[scope] as unknown as TokenStyle)
      }
    })

    const tokenStyles: TokenStyle = {}
    if (token.color) {
      tokenStyles.color = token.color
    }
    if (token.fontStyle === 'italic') {
      tokenStyles.fontStyle = 'italic'
    }
    if (token.fontWeight === 'bold') {
      tokenStyles.fontWeight = 'bold'
    }

    if (Object.keys(tokenStyles).length > 0) {
      styles.push(tokenStyles)
    }

    const combinedStyles = StyleSheet.flatten(styles)

    return (
      <Text key={`${lineIndex}-${index}`} style={combinedStyles}>
        {token.content.replace(/ /g, '\u00A0')}
      </Text>
    )
  }

  const renderLine = (line: any[], lineIndex: number) => (
    <View key={lineIndex} style={baseStyles.lineContainer}>
      {line.map((token, tokenIndex) => renderToken(token, tokenIndex, lineIndex))}
    </View>
  )

  return (
    <ScrollView horizontal showsHorizontalScrollIndicator={Platform.OS !== 'web'} style={[baseStyles.scrollView, stylesheet.base as unknown as TokenStyle]}>
      <View style={baseStyles.container} onStartShouldSetResponder={() => true}>
        {tokens.map((line, index) => renderLine(line, index))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
